package org.belot.client;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import org.belot.protocol.*;
import org.belot.server.*;

import java.io.*;
import java.util.*;
import java.net.*;

import org.belot.client.board.*;
import org.belot.client.images.*;

public abstract class BelotClient
extends JFrame
implements WindowListener, ActionListener {

  protected JTabbedPane tabbedPane = new JTabbedPane();

  protected JTextArea textArea = new JTextArea();
  protected JScrollPane textAreaScroll = new JScrollPane(textArea);
  protected JTextField messageField = new JTextField();
  protected JButton sendMessageButton = new JButton("Send");
  protected JButton getPlayersButton = new JButton("Get Players");
  protected JButton clearButton = new JButton("Clear All");

  protected String[] positionsArray = new String[] {
    "West",
    "South",
    "East",
    "North",
    "N/A",
  };

  protected JLabel gameModeLabel = new JLabel("Unknown Game");
  protected JLabel playerTurnLabel = new JLabel("Unknown Turn");
  protected JLabel trumpLabel = new JLabel("Unknown Trump");

  protected JLabel nameTextLabel = new JLabel(BelotProtocol.AnonymousCoward);
  protected JTextField nameTextField = new JTextField();
  protected JButton nameChangeButton = new JButton("Change Name");

  protected JLabel positionTextLabel = new JLabel(positionsArray[4]);
  protected JComboBox positionComboBox = new JComboBox(positionsArray);
  protected JButton positionChangeButton = new JButton("Change Position");

  protected Font textFont = new Font("Monospaced", Font.PLAIN, 12);
  protected Font textBoldFont = new Font("Monospaced", Font.BOLD, 12);

  protected JLabel serverLabel = new JLabel("Server: ");
  protected JLabel portLabel = new JLabel("Port: ");
  protected JTextField serverTextField = new JTextField();
  protected JTextField portTextField = new JTextField();
  protected JButton connectButton = new JButton("Connect");
  protected JButton disconnectButton = new JButton("Disconnect");

  protected JMenuItem playButton = new BelotMenuItem("Play");
  protected JMenuItem playBelotButton = new BelotMenuItem("Play Belot");

  protected JMenuItem announce3SeqButton = new BelotMenuItem("Announce Terza");
  protected JMenuItem announce4SeqButton = new BelotMenuItem("Announce 50");
  protected JMenuItem announce5SeqButton = new BelotMenuItem("Announce 100 of Cards");
  protected JMenuItem announceFourButton = new BelotMenuItem("Announce Four");

  protected JMenuItem contraButton = new BelotMenuItem("Contra");
  protected JMenuItem reContraButton = new BelotMenuItem("Re-Contra");
  protected JMenuItem allTrumpButton = new BelotMenuItem("All Trump");
  protected JMenuItem noTrumpButton = new BelotMenuItem("No Trump");
  protected JMenuItem spadesButton = new BelotMenuItem("Spades");
  protected JMenuItem heartsButton = new BelotMenuItem("Hearts");
  protected JMenuItem diamondsButton  = new BelotMenuItem("Diamonds");
  protected JMenuItem clubsButton = new BelotMenuItem("Clubs");
  protected JMenuItem passButton = new BelotMenuItem("Pass");

  protected JMenuItem startGameButton = new BelotMenuItem("Start Game");
  protected JMenuItem startDealingButton = new BelotMenuItem("Start Dealing");

  protected JMenuItem cancelButton = new BelotMenuItem("Cancel");

  protected JPopupMenu popupMenu = new JPopupMenu();

  protected JLabel teamOneScoreLabel = new JLabel();
  protected JLabel teamTwoScoreLabel = new JLabel();

  protected JLabel dealOneScoreLabel = new JLabel();
  protected JLabel dealTwoScoreLabel = new JLabel();

  protected JLabel hangingLabel = new JLabel();

  protected CardsImages cardsImages = new CardsImages();
  protected HandPanel handPanel;
  protected CardsPanel cardsPanel;
  protected JLabel[] namesLabels = new JLabel[] {
    new JLabel(),
    new JLabel(),
    new JLabel(),
    new JLabel(),
  };
  protected JLabel[] announcesLabels = new JLabel[] {
    new JLabel(),
    new JLabel(),
    new JLabel(),
    new JLabel(),
  };

  public BelotClient() {
    super("Belot Client");

    handPanel = new HandPanelImpl(cardsImages, this);
    cardsPanel = new CardsPanelImpl(cardsImages, this);

    textArea.setFont(textBoldFont);
    messageField.setFont(textFont);
    textArea.setEditable(false);
    textArea.setWrapStyleWord(true);

    gameModeLabel.setFont(textBoldFont);
    playerTurnLabel.setFont(textBoldFont);
    trumpLabel.setFont(textBoldFont);

    nameTextLabel.setFont(textBoldFont);
    nameTextField.setFont(textFont);
    nameTextField.setText(nameTextLabel.getText());

    positionTextLabel.setFont(textBoldFont);
    positionComboBox.setEditable(false);
    positionComboBox.setSelectedIndex(4);
    positionComboBox.setFont(textFont);

    serverLabel.setFont(textBoldFont);
    portLabel.setFont(textBoldFont);
    serverTextField.setFont(textFont);
    portTextField.setFont(textFont);
    serverTextField.setText("localhost");
    portTextField.setText(Integer.toString(BelotProtocol.BelotPort));

    setButtonSize(sendMessageButton);
    sendMessageButton.setMnemonic('S');
    sendMessageButton.addActionListener(this);

    setButtonSize(getPlayersButton);
    getPlayersButton.setMnemonic('P');
    getPlayersButton.addActionListener(this);

    setButtonSize(clearButton);
    clearButton.addActionListener(this);

    setButtonSize(nameChangeButton);
    setButtonSize(positionChangeButton);

    setButtonSize(connectButton);
    setButtonSize(disconnectButton);

    spadesButton.setIcon(cardsImages.getImage(BelotColor.Spades));
    heartsButton.setIcon(cardsImages.getImage(BelotColor.Hearts));
    diamondsButton.setIcon(cardsImages.getImage(BelotColor.Diamonds));
    clubsButton.setIcon(cardsImages.getImage(BelotColor.Clubs));

    playButton.addActionListener(this);
    playBelotButton.addActionListener(this);

    announce3SeqButton.addActionListener(this);
    announce4SeqButton.addActionListener(this);
    announce5SeqButton.addActionListener(this);
    announceFourButton.addActionListener(this);

    contraButton.addActionListener(this);
    reContraButton.addActionListener(this);
    allTrumpButton.addActionListener(this);
    noTrumpButton.addActionListener(this);
    spadesButton.addActionListener(this);
    heartsButton.addActionListener(this);
    diamondsButton.addActionListener(this);
    clubsButton.addActionListener(this);
    passButton.addActionListener(this);

    startGameButton.addActionListener(this);
    startDealingButton.addActionListener(this);

    cancelButton.addActionListener(this);

    popupMenu.add(announce3SeqButton);
    popupMenu.add(announce4SeqButton);
    popupMenu.add(announce5SeqButton);
    popupMenu.add(announceFourButton);

    popupMenu.add(playBelotButton);
    popupMenu.add(playButton);

    popupMenu.add(contraButton);
    popupMenu.add(reContraButton);
    popupMenu.add(allTrumpButton);
    popupMenu.add(noTrumpButton);
    popupMenu.add(spadesButton);
    popupMenu.add(heartsButton);
    popupMenu.add(diamondsButton);
    popupMenu.add(clubsButton);
    popupMenu.add(passButton);

    popupMenu.add(startGameButton);
    popupMenu.add(startDealingButton);

    popupMenu.add(cancelButton);

    Insets insets = new Insets(5, 5, 5, 5);
    Insets nullInsets = new Insets(0, 0, 0, 0);

    addWindowListener(this);
    Dimension size = new Dimension(850, 600);
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    setLocation((screenSize.width - size.width) / 2, (screenSize.height - size.height) / 2);
    setSize(size);

    JPanel connectionPanel = new JPanel(new GridBagLayout());
    {
      JPanel panel = new JPanel(new GridBagLayout());
      panel.setBorder(new TitledBorder(new EtchedBorder(), "Name"));
      panel.add(nameTextLabel, new GridBagConstraints(0, 0, 2, 1, 1.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
      panel.add(nameTextField, new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
      panel.add(nameChangeButton, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
      nameChangeButton.addActionListener(this);
      connectionPanel.add(panel, new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    }
    {
      JPanel panel = new JPanel(new GridBagLayout());
      panel.setBorder(new TitledBorder(new EtchedBorder(), "Position"));
      panel.add(positionTextLabel, new GridBagConstraints(0, 0, 2, 1, 1.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
      panel.add(positionComboBox, new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
      panel.add(positionChangeButton, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
      positionChangeButton.addActionListener(this);
      connectionPanel.add(panel, new GridBagConstraints(0, 2, 1, 1, 1.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    }
    {
      JPanel panel = new JPanel(new GridBagLayout());
      panel.setBorder(new TitledBorder(new EtchedBorder(), "Server"));
      panel.add(serverLabel, new GridBagConstraints(0, 0, 2, 1, 1.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, insets, 0, 0));
      panel.add(serverTextField, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
      panel.add(portLabel, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, insets, 0, 0));
      panel.add(portTextField, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
      panel.add(connectButton, new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
      panel.add(disconnectButton, new GridBagConstraints(0, 3, 2, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
      connectButton.addActionListener(this);
      disconnectButton.addActionListener(this);
      connectionPanel.add(panel, new GridBagConstraints(0, 3, 1, 1, 1.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    }
    JPanel gamePanel = new JPanel(new GridBagLayout());
    {
      int y = 0;
      JPanel modeLabelsPanel = new JPanel(new GridBagLayout());
      modeLabelsPanel.add(gameModeLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
      modeLabelsPanel.add(playerTurnLabel, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
      JPanel trumpLabelPanel = new JPanel(new GridBagLayout());
      trumpLabelPanel.add(trumpLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
      JPanel labelsPanel = new JPanel(new BorderLayout());
      labelsPanel.add(modeLabelsPanel, BorderLayout.WEST);
      labelsPanel.add(trumpLabelPanel, BorderLayout.EAST);
      gamePanel.add(labelsPanel, new GridBagConstraints(0, y++, 1, 1, 1.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));

      class Listener extends MouseAdapter {
        public void mousePressed(MouseEvent e) {
          BelotClient.this.showPopupMenu((JComponent)e.getSource(), e.getX(), e.getY());
          e.consume();
        }
      }

      Listener listener = new Listener();
      JPanel handPanelAnnounces = new JPanel(new GridBagLayout());
      {
        handPanelAnnounces.addMouseListener(listener);
        JPanel[] panels = new JPanel[4];
        for (int i = 0; i < 4; ++i) {
          panels[i] = new JPanel(new GridBagLayout());
          fixLabel(announcesLabels[i], listener, false);
          fixLabel(namesLabels[i], listener, true);
          panels[i].addMouseListener(listener);
          panels[i].add(namesLabels[i], new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.NONE, nullInsets, 0, 0));
          panels[i].add(announcesLabels[i], new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.NONE, nullInsets, 0, 0));
        }
        handPanelAnnounces.add(panels[3], new GridBagConstraints(0, 0, 3, 1, 0.0, 0.0,
          GridBagConstraints.CENTER, GridBagConstraints.NONE, nullInsets, 0, 0));
        handPanelAnnounces.add(panels[0], new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
          GridBagConstraints.EAST, GridBagConstraints.NONE, nullInsets, 0, 0));
        handPanelAnnounces.add(handPanel, new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0,
          GridBagConstraints.CENTER, GridBagConstraints.BOTH, insets, 0, 0));
        handPanelAnnounces.add(panels[2], new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0,
          GridBagConstraints.WEST, GridBagConstraints.NONE, nullInsets, 0, 0));
        handPanelAnnounces.add(panels[1], new GridBagConstraints(0, 2, 3, 1, 0.0, 0.0,
          GridBagConstraints.CENTER, GridBagConstraints.NONE, nullInsets, 0, 0));
      }
      gamePanel.add(new JScrollPane(handPanelAnnounces), new GridBagConstraints(0, y++, 1, 1, 1.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.BOTH, insets, 0, 0));
      JScrollPane pane = new JScrollPane(cardsPanel);
      Dimension cardsPanelSize = cardsPanel.getPreferredSize();
      Dimension paneSize = new Dimension(cardsPanelSize.width + 10, cardsPanelSize.height + 10);
      pane.setPreferredSize(paneSize);
      pane.setMinimumSize(paneSize);
      gamePanel.add(pane, new GridBagConstraints(0, y++, 1, 1, 1.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
    }
    JPanel scoresPanel = new JPanel(new GridBagLayout());
    {
      scoresPanel.add(teamOneScoreLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
      scoresPanel.add(teamTwoScoreLabel, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
      scoresPanel.add(dealOneScoreLabel, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
      scoresPanel.add(dealTwoScoreLabel, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
      scoresPanel.add(hangingLabel, new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
    }
    JPanel chatPanel = new JPanel(new GridBagLayout());
    {
      chatPanel.add(textAreaScroll, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.BOTH, insets, 0, 0));
      JPanel messagePanel = new JPanel(new BorderLayout(5, 5));
      messagePanel.add(messageField, BorderLayout.CENTER);
      messagePanel.add(sendMessageButton, BorderLayout.EAST);
      chatPanel.add(messagePanel, new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, insets, 0, 0));
      JPanel lowerPanel = new JPanel(new GridBagLayout());
      lowerPanel.add(clearButton, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
      lowerPanel.add(getPlayersButton, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
      chatPanel.add(lowerPanel, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
        GridBagConstraints.CENTER, GridBagConstraints.NONE, nullInsets, 0, 0));
    }
    JPanel mainPanel = new JPanel(new GridBagLayout());
    {
      mainPanel.add(tabbedPane, new GridBagConstraints(0, 1, 2, 1, 1.0, 1.0,
        GridBagConstraints.CENTER, GridBagConstraints.BOTH, insets, 0, 0));
      tabbedPane.add("Connection", connectionPanel);
      tabbedPane.add("Game", gamePanel);
      tabbedPane.add("Score", scoresPanel);
      tabbedPane.add("Chat", chatPanel);
    }
    getContentPane().add(mainPanel);
  }

  protected void fixLabel(JLabel label, MouseListener listener, boolean isLong) {
    label.setHorizontalAlignment(JLabel.CENTER);
    label.setVerticalAlignment(JLabel.CENTER);
    label.setFont(textBoldFont);
    if (isLong) {
      setAllSizes(label, new Dimension(200, 16));
    } else {
      setAllSizes(label, new Dimension(70, 16));
    }
    label.addMouseListener(listener);
  }

  protected void setAllSizes(JComponent button, Dimension size) {
    button.setSize(size);
    button.setPreferredSize(size);
    button.setMinimumSize(size);
    button.setMaximumSize(size);
  }

  protected void setButtonSize(JComponent button) {
    Dimension size = new Dimension(130, 25);
    setAllSizes(button, size);
  }

  public void windowClosing(WindowEvent e) {
    finishProgram();
  }
  public void windowClosed(WindowEvent e) {
    finishProgram();
  }

  private static BelotClient belotClientInstance;

  public static void main(String[] arg) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClientInstance = new BelotClientImpl();
        belotClientInstance.fixEnabledState();
        belotClientInstance.setVisible(true);
      }
    });
  }

  public void windowOpened(WindowEvent e) {
    // do nothing.
  }
  public void windowIconified(WindowEvent e) {
    // do nothing.
  }
  public void windowDeiconified(WindowEvent e) {
    // do nothing.
  }
  public void windowDeactivated(WindowEvent e) {
    // do nothing.
  }
  public void windowActivated(WindowEvent e) {
    // do nothing.
  }

  public abstract void actionPerformed(ActionEvent e);
  public abstract void finishProgram();
  public abstract void fixEnabledState();
  public abstract void showPopupMenu(JComponent parent, int x, int y);

}