package org.belot.client;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import org.belot.protocol.*;
import org.belot.server.*;

import java.io.*;
import java.util.*;
import java.net.*;

import org.belot.client.board.*;

public class BelotClientImpl extends BelotClient {

  private static final int TIMEOUT = 3000;

  // cards:
  private BelotCard[] cards = new BelotCard[8];
  private BelotCard[] sortedCards = new BelotCard[0];
  private CardsComparator cardsComparator = new CardsComparator();
  private Announces announces = new Announces();
  private long[] announcesTimer = new long[4];

  // players:
  private BelotClientPlayer[] activePlayers = new BelotClientPlayer[4];

  // hand:
  private BelotCard[] handOfCards = new BelotCard[4];
  private int firstHandPlayer = -10;
  private long handFullTimer = 0;

  // trump info:
  private BelotColor trump = null;
  private boolean isContra;
  private boolean isReContra;
  private int bettingPlayer;

  // game mode:
  private int gameMode = -10;

  // player turn:
  private int playerTurn = -10;

  // connection:
  private Socket socket;
  private ClientSenderThread clientSenderThread;
  private BelotClientReader belotClientReader;

  // timer:
  private javax.swing.Timer timer;

  public BelotClientImpl() {
    super();
    this.timer = new javax.swing.Timer(1000, new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            doTimer();
          }
        });
      }
    });
    timer.start();
  }

  public void finishProgram() {
    System.out.println(getSize());
    System.exit(0);
  }

  private String getDirection(int direction) {
    if (direction < 0) direction = 4;
    if (direction > 4) direction = 4;
    return positionsArray[direction];
  }

  public void listPlayers(Vector players) {
    int size = players.size();
    for (int i = 0; i < size; ++i) {
      BelotClientPlayer player = (BelotClientPlayer)players.elementAt(i);
      listPlayer(player);
    }
  }

  private static boolean findDuplicate(int index, String[] names) {
    for (int i = 0; i < names.length; ++i) {
      if (i != index && names[i].equals(names[index])) {
        return true;
      }
    }
    return false;
  }

  public void setActivePlayers(BelotClientPlayer[] players) {
    String[] names = new String[4];
    for (int i = 0; i < 4; ++i) {
      names[i] = players[i].getName() + ": " + getDirection(players[i].getDirection());
    }
    String[] changedNames = new String[4];
    for (int i = 0; i < 4; ++i) {
      if (findDuplicate(i, names)) {
        changedNames[i] = names[i] + " (" + players[i].getId() + ")";
      } else {
        changedNames[i] = names[i];
      }
    }
    names = changedNames;
    for (int i = 0; i < 4; ++i) {
      System.out.println("players["+i+"] = " + players[i].getDirection());
      if (players[i].getId() < 0) {
        int index = getPlayerIndex(i);
        namesLabels[index].setForeground(Color.red);
        namesLabels[index].setText("Nobody");
      } else {
        int index = getPlayerIndex(players[i].getDirection());
        namesLabels[index].setForeground(Color.black);
        namesLabels[index].setText(names[i]);
      }
    }
  }

  private int getPlayerIndex(int playerDirection) {
    int currentDirection = directionIndex(positionTextLabel.getText().trim());
    return (playerDirection + (HandPanelImpl.SOUTH - currentDirection + 16)) % 4;
  }

  public void listPlayer(BelotClientPlayer player) {
    textArea.append(
      "Player " + player.getId() +
      " '" + player.getName() +
      "' at " + getDirection(player.getDirection()) +
      System.getProperty("line.separator"));
  }

  public void removePlayer(BelotClientPlayer player) {
    textArea.append(
      "Player " + player.getId() +
      " '" + player.getName() +
      "' at " + getDirection(player.getDirection()) +
      " has quit" +
      System.getProperty("line.separator"));
  }

  private String getColorText(int color) {
    if (color == BelotColor.COLOR_ALLTRUMP) {
      return "All Trump";
    }
    if (color == BelotColor.COLOR_NOTRUMP) {
      return "No Trump";
    }
    if (color == BelotColor.COLOR_CLUBS) {
      return "Clubs";
    }
    if (color == BelotColor.COLOR_DIAMONDS) {
      return "Diamonds";
    }
    if (color == BelotColor.COLOR_HEARTS) {
      return "Hearts";
    }
    if (color == BelotColor.COLOR_SPADES) {
      return "Spades";
    }
    return "Pass";
  }

  private String getBet(int color, boolean contra, boolean recontra, boolean nothing) {
    String colorText = getColorText(color);
    if (nothing && "Pass".equals(colorText)) {
      colorText = "Nothing";
    } else {
      if (recontra) {
        colorText += ", Re-Contra";
      } else if (contra) {
        colorText += ", Contra";
      }
    }
    return colorText;
  }

  public void setPlayerBet(BelotClientPlayer player, int color, boolean contra, boolean reContra) {
    textArea.append(
      "Player " + player.getId() +
      " '" + player.getName() +
      "' at " + getDirection(player.getDirection()) +
      " bets " + getBet(color, contra, reContra, false) +
      System.getProperty("line.separator"));
    int index = getPlayerIndex(player.getDirection());
    announcesTimer[index] = System.currentTimeMillis();
    announcesLabels[index].setText(getBet(color, contra, reContra, false));
  }

  public void setPlayerMessageLine(BelotClientPlayer player, String line) {
    textArea.append(
      "Player " + player.getId() +
      " '" + player.getName() +
      "' says: " + line +
      System.getProperty("line.separator"));
  }

  public void setPlayerPlay(BelotClientPlayer player, BelotCard card) {
    textArea.append(
      "Player " + player.getId() +
      " '" + player.getName() +
      "' at " + getDirection(player.getDirection()) +
      " plays " + card +
      System.getProperty("line.separator"));
  }

  public void setPlayerPlayBelot(BelotClientPlayer player, BelotCard card) {
    textArea.append(
      "Player " + player.getId() +
      " '" + player.getName() +
      "' at " + getDirection(player.getDirection()) +
      " plays " + card + ", Belot" +
      System.getProperty("line.separator"));
    int index = getPlayerIndex(player.getDirection());
    announcesTimer[index] = System.currentTimeMillis();
    announcesLabels[index].setText("Belot");
  }

  private String getAnnounce(int what) {
    switch (what) {
      case (BelotProtocol.AnnounceSequence3) : return "Terza";
      case (BelotProtocol.AnnounceSequence4) : return "50";
      case (BelotProtocol.AnnounceSequence5) : return "100";
      case (BelotProtocol.AnnounceFourOther) : return "100";
      case (BelotProtocol.AnnounceFourNines) : return "150";
      case (BelotProtocol.AnnounceFourJacks) : return "200";
    }
    return "700";
  }

  public void setPlayerAnnounce(BelotClientPlayer player, int what) {
    textArea.append(
      "Player " + player.getId() +
      " '" + player.getName() +
      "' at " + getDirection(player.getDirection()) +
      " announces " + getAnnounce(what) +
      System.getProperty("line.separator"));
    int index = getPlayerIndex(player.getDirection());
    announcesTimer[index] = System.currentTimeMillis();
    announcesLabels[index].setText(getAnnounce(what));
  }

  private String getGameMode(int mode) {
    String result = "Unknown";
    switch (mode) {
      case (GameManager.BETTING) : {
        result = "Betting";
        break;
      } case (GameManager.NO_DEALING) : {
        result = "No Dealing";
        break;
      } case (GameManager.NO_GAME) : {
        result = "No Game";
        break;
      } case (GameManager.PLAYING) : {
        result = "Playing";
        break;
      }
    }
    return result;
  }

  private int directionIndex(String direction) {
    if (direction == null) return -1;
    for (int i = 0; i < positionsArray.length; ++i) {
      if (direction.equals(positionsArray[i])) {
        return i;
      }
    }
    return -1;
  }

  private void getCards() {
    clientSenderThread.depositMessage("" + belotClientReader.nextGetCardsId() + ":" + BelotProtocol.GetCards);
  }

  public void gameMode(int mode, int playerTurn) {
    System.out.println("Setting Game Mode: " + mode + ", playerTurn = " + playerTurn);
    if (this.gameMode != mode) {
      clientSenderThread.depositMessage(":" + BelotProtocol.GetCards);
    }
    gameModeLabel.setText(getGameMode(mode));
    boolean isTurnOK = (playerTurn >= 0 && playerTurn <= 3) &&
      (mode == GameManager.PLAYING || mode == GameManager.BETTING);
    if (isTurnOK) {
      playerTurnLabel.setText(getDirection(playerTurn));
    } else {
      playerTurnLabel.setText("Unknown Turn");
    }
    this.playerTurn = playerTurn;
    if (this.playerTurn < 0 || this.playerTurn > 3) {
      this.playerTurn = -10;
    }
    if (isTurnOK && this.playerTurn >= 0) {
      int playerTurnIndex = getPlayerIndex(this.playerTurn);
      for (int i = 0; i < 4; ++i) {
        if (playerTurnIndex == i) {
          namesLabels[i].setIcon(cardsImages.getBall());
        } else {
          namesLabels[i].setIcon(null);
        }
      }
    } else {
      for (int i = 0; i < 4; ++i) {
        namesLabels[i].setIcon(null);
      }
    }
    this.gameMode = mode;
    if (mode != GameManager.PLAYING && mode != GameManager.BETTING) {
      trump = null;
      announces.clear();
    }
    System.out.println("Setting Game Mode Ended: " + this.gameMode + ", playerTurn = " + this.playerTurn);
  }

  public void setCards(BelotCard[] cards) {
    this.cards = cards;
    Vector vectorCards = new Vector(8);
    for (int i = 0; i < 8; ++i) {
      if (cards[i] != null) {
        vectorCards.addElement(cards[i]);
      }
    }
    cardsComparator.setTrump(trump);
    HeapSort.heapSort(vectorCards, cardsComparator);
    this.sortedCards = new BelotCard[vectorCards.size()];
    for (int i = 0; i < this.sortedCards.length; ++i) {
      this.sortedCards[i] = (BelotCard)vectorCards.elementAt(i);
    }
    this.cardsPanel.clearSelection();
    this.cardsPanel.loadCards(sortedCards);
  }

  public void setHand(BelotCard[] hand, int firstHandPlayer) {
    if (hand[3] != null) {
      handFullTimer = System.currentTimeMillis();
    }
    this.handOfCards = hand;
    this.firstHandPlayer = firstHandPlayer;
  }

  public void setTrump(BelotColor trump, boolean contra, boolean recontra, int player) {
    this.trump = trump;
    this.isContra = contra;
    this.isReContra = recontra;
    this.bettingPlayer = player;
    textArea.append(
      "Current trump is '" + getBet(trump != null ? trump.getColor() : -1, contra, recontra, true) +
      "' bet by player at " + getDirection(player) +
      System.getProperty("line.separator"));
    setCards(this.cards);
  }

  public void setScores(int[] dealingScores, int[] teamScores) {
    this.teamOneScoreLabel.setText(Integer.toString(teamScores[0]));
    this.teamTwoScoreLabel.setText(Integer.toString(teamScores[1]));
    this.dealOneScoreLabel.setText(Integer.toString(dealingScores[0]));
    this.dealTwoScoreLabel.setText(Integer.toString(dealingScores[1]));
    this.hangingLabel.setText(Integer.toString(dealingScores[2]));
  }

  private int getDirection(String text) {
    for (int i = 0; i < positionsArray.length; ++i) {
      if (text.equals(positionsArray[i])) {
        return i;
      }
    }
    return -1;
  }

  private void setName() {
    String message = ":" + BelotProtocol.Name + ":" + nameTextLabel.getText().trim();
    System.out.println(message);
    clientSenderThread.depositMessage(message);
  }

  private void setDirection() {
    {
      String message = ":" + BelotProtocol.Direction + ":" + getDirection(positionTextLabel.getText().trim());
      clientSenderThread.depositMessage(message);
    }
    {
      clientSenderThread.depositMessage(":" + BelotProtocol.GetCards);
    }
  }

  private void betContra() {
    if (clientSenderThread != null) {
      BelotColor betting = trump;
      clientSenderThread.depositMessage(":" + BelotProtocol.TrumpChoice + ":" +
        (betting != null ? betting.getColor() : -1) + ":" + true);
      clientSenderThread.depositMessage(":" + BelotProtocol.GetCards);
    }
  }

  private void bet(BelotColor betting) {
    if (clientSenderThread != null) {
      clientSenderThread.depositMessage(":" + BelotProtocol.TrumpChoice + ":" +
        (betting != null ? betting.getColor() : -1) + ":" + false);
      clientSenderThread.depositMessage(":" + BelotProtocol.GetCards);
    }
  }

  public void disconnect() {
    if (clientSenderThread != null) {
      clientSenderThread.closeClientSenderThread();
      clientSenderThread = null;
    }
    if (socket != null) {
      try {
        socket.close();
      } catch (Throwable error) {
        error.printStackTrace(System.out);
      }
      socket = null;
    }
    belotClientReader = null;
    gameMode = -10;
  }

  private void cleanGameState(int gameMode) {
    // now, fix the game state:
    cards = new BelotCard[8];
    sortedCards = new BelotCard[0];
    cardsComparator = new CardsComparator();

    // game mode:
    this.gameMode = gameMode;

    // player turn:
    playerTurn = -10;

    // fix the UI:
    cardsPanel.clearSelection();
    cardsPanel.loadCards(sortedCards);
  }

  private int findCard(BelotCard card) {
    int index = -1;
    for (int i = 0; i < cards.length; ++i) {
      if (card == cards[i]) {
        index = i;
        break;
      }
    }
    return index;
  }

  private void announce(int what) {
    BelotCard selectedCard = (BelotCard)cardsPanel.getSelectedValue();
    int index = findCard(selectedCard);
    announces.add(what, cards, index);
    clientSenderThread.depositMessage(
      ":" + BelotProtocol.Announce + ":" + what + ":" + index);
  }

  public void actionPerformed(ActionEvent e) {
    try {
      Object source = e.getSource();
      if (source == nameChangeButton) {
        nameTextLabel.setText(nameTextField.getText().trim());
        if (clientSenderThread != null) {
          setName();
        }
      } else if (source == positionChangeButton) {
        positionTextLabel.setText(positionsArray[positionComboBox.getSelectedIndex()]);
        if (clientSenderThread != null) {
          setDirection();
        }
      } else if (source == connectButton) {
        try {
          socket = new Socket(serverTextField.getText().trim(), Integer.parseInt(portTextField.getText().trim()));
          clientSenderThread = new ClientSenderThread(socket.getOutputStream());
          setName();
          setDirection();
          belotClientReader = new BelotClientReader(
            this, new LineNumberReader(new InputStreamReader(socket.getInputStream(), "UTF8")));
          clientSenderThread.depositMessage(":" + BelotProtocol.GetCards);
          textArea.setText("---Connected---" + System.getProperty("line.separator"));
        } catch (Throwable error) {
          error.printStackTrace(System.out);
          disconnect();
        }
      } else if (source == disconnectButton) {
        disconnect();
        textArea.setText("---Disconnected---" + System.getProperty("line.separator"));
      } else if (source == playButton) {
        BelotCard selectedCard = (BelotCard)cardsPanel.getSelectedValue();
        if (clientSenderThread != null && selectedCard != null) {
          int index = findCard(selectedCard);
          clientSenderThread.depositMessage(":" + BelotProtocol.Play + ":" + index);
          clientSenderThread.depositMessage(":" + BelotProtocol.GetCards);
        }
      } else if (source == playBelotButton) {
        BelotCard selectedCard = (BelotCard)cardsPanel.getSelectedValue();
        int index = findCard(selectedCard);
        clientSenderThread.depositMessage(":" + BelotProtocol.PlayBelot + ":" + index);
        clientSenderThread.depositMessage(":" + BelotProtocol.GetCards);
      } else if (source == contraButton) {
        betContra();
      } else if (source == reContraButton) {
        betContra();
      } else if (source == allTrumpButton) {
        bet(BelotColor.AllTrump);
      } else if (source == noTrumpButton) {
        bet(BelotColor.NoTrump);
      } else if (source == spadesButton) {
        bet(BelotColor.Spades);
      } else if (source == heartsButton) {
        bet(BelotColor.Hearts);
      } else if (source == diamondsButton) {
        bet(BelotColor.Diamonds);
      } else if (source == clubsButton) {
        bet(BelotColor.Clubs);
      } else if (source == passButton) {
        bet(null);
      } else if (source == startGameButton) {
        if (clientSenderThread != null) {
          clientSenderThread.depositMessage(":" + BelotProtocol.StartGame);
        }
      } else if (source == startDealingButton) {
        if (clientSenderThread != null) {
          clientSenderThread.depositMessage(":" + BelotProtocol.StartDealing);
        }
      } else if (source == announce3SeqButton) {
        announce(BelotProtocol.AnnounceSequence3);
      } else if (source == announce4SeqButton) {
        announce(BelotProtocol.AnnounceSequence4);
      } else if (source == announce5SeqButton) {
        announce(BelotProtocol.AnnounceSequence5);
      } else if (source == announceFourButton) {
        int what = getWhatFour();
        announce(what);
      } else if (source == cancelButton) {
        popupMenu.setVisible(false);
      } else if (source == sendMessageButton) {
        if (clientSenderThread != null) {
          String text = messageField.getText().trim();
          if (text.length() == 0) {
            textArea.append("Any other enlightening remarks?" + System.getProperty("line.separator"));
          } else {
            clientSenderThread.depositMessage(":" + BelotProtocol.Say + ":" + text);
            messageField.setText("");
            messageField.requestFocus();
          }
        }
      } else if (source == clearButton) {
        textArea.setText("---Cleared---" + System.getProperty("line.separator"));
      } else if (source == getPlayersButton) {
        if (clientSenderThread != null) {
          clientSenderThread.depositMessage(":" + BelotProtocol.ListPlayers);
        }
      }
    } finally {
      fixEnabledState();
    }
  }

  private int getWhatFour() {
    BelotCard selectedCard = (BelotCard)cardsPanel.getSelectedValue();
    if (selectedCard == null) {
      return -1;
    }
    int what = BelotProtocol.AnnounceFourOther;
    if (selectedCard.getNumber() == BelotCard.NUMBER_JACK) {
      what = BelotProtocol.AnnounceFourJacks;
    } else if (selectedCard.getNumber() == 9) {
      what = BelotProtocol.AnnounceFourNines;
    }
    return what;
  }

  public void showPopupMenu(JComponent parent, int x, int y) {
    fixPlayButton();
    fixAnnounces();
    fixBets();
    popupMenu.show(parent, x, y);
  }

  private void fixAnnounces() {
    int currentDirection = directionIndex(positionTextLabel.getText().trim());
    boolean canPlay = (trump != BelotColor.NoTrump) &&
      clientSenderThread != null && gameMode == GameManager.PLAYING && currentDirection == playerTurn;
    BelotCard selectedCard = (BelotCard)cardsPanel.getSelectedValue();
    int index = findCard(selectedCard);
    canPlay = canPlay && selectedCard != null;
    announce3SeqButton.setEnabled(canPlay && announces.canAdd(BelotProtocol.AnnounceSequence3, cards, index));
    announce4SeqButton.setEnabled(canPlay && announces.canAdd(BelotProtocol.AnnounceSequence4, cards, index));
    announce5SeqButton.setEnabled(canPlay && announces.canAdd(BelotProtocol.AnnounceSequence5, cards, index));
    announceFourButton.setEnabled(canPlay && announces.canAdd(getWhatFour(), cards, index));
  }

  private void fixPlayButton() {
    int currentDirection = directionIndex(positionTextLabel.getText().trim());
    boolean canPlay = clientSenderThread != null && gameMode == GameManager.PLAYING && currentDirection == playerTurn;
    canPlay = canPlay && cardsPanel.getSelectedValue() != null;
    playButton.setEnabled(canPlay);
    playBelotButton.setEnabled(canPlay && BelotGameImpl.canHaveBelot(
      handOfCards,
      trump,
      cards,
      findCard((BelotCard)cardsPanel.getSelectedValue())));
  }

  private void fixScore(JLabel label, String text) {
    String oldText = label.getText().trim();
    if (oldText.length() == 0) {
      oldText = "0";
    }
    if (!oldText.startsWith(text)) {
      label.setText(text + ": " + oldText);
    }
  }

  private void fixBets() {
    int currentDirection = directionIndex(positionTextLabel.getText().trim());
    boolean canBet = clientSenderThread != null && gameMode == GameManager.BETTING && currentDirection == playerTurn;
    if (canBet) {
      contraButton.setEnabled(
        (trump != null) &&
        (!isContra) &&
        (!isReContra) &&
        (BelotGameImpl.getTeamOfPlayer(currentDirection) != BelotGameImpl.getTeamOfPlayer(bettingPlayer))
      );
      reContraButton.setEnabled(
        (trump != null) &&
        (isContra) &&
        (!isReContra) &&
        (BelotGameImpl.getTeamOfPlayer(currentDirection) != BelotGameImpl.getTeamOfPlayer(bettingPlayer))
      );
      allTrumpButton.setEnabled(trump == null || trump.compareTo(BelotColor.AllTrump) < 0);
      noTrumpButton.setEnabled(trump == null || trump.compareTo(BelotColor.NoTrump) < 0);
      spadesButton.setEnabled(trump == null || trump.compareTo(BelotColor.Spades) < 0);
      heartsButton.setEnabled(trump == null || trump.compareTo(BelotColor.Hearts) < 0);
      diamondsButton.setEnabled(trump == null || trump.compareTo(BelotColor.Diamonds) < 0);
      clubsButton.setEnabled(trump == null || trump.compareTo(BelotColor.Clubs) < 0);
      passButton.setEnabled(true);
    } else {
      contraButton.setEnabled(false);
      reContraButton.setEnabled(false);
      allTrumpButton.setEnabled(false);
      noTrumpButton.setEnabled(false);
      spadesButton.setEnabled(false);
      heartsButton.setEnabled(false);
      diamondsButton.setEnabled(false);
      clubsButton.setEnabled(false);
      passButton.setEnabled(false);
    }
  }

  private void fixHand() {
    int currentDirection = directionIndex(positionTextLabel.getText().trim());
    handPanel.loadCards(handOfCards, currentDirection, firstHandPlayer);
  }

  private void fixTrump() {
    trumpLabel.setIcon(cardsImages.getImage(trump));
    String text =
      "" +
      getBet(trump != null ? trump.getColor() : -1, isContra, isReContra, true) +
      ", " +
      (trump != null ? getDirection(bettingPlayer) : "Nobody");
    trumpLabel.setText(text);
  }

  public void fixEnabledState() {
    sendMessageButton.setEnabled(clientSenderThread != null);
    getPlayersButton.setEnabled(clientSenderThread != null);
    fixHand();
    fixTrump();
    fixScore(teamOneScoreLabel, "Team One Total");
    fixScore(teamTwoScoreLabel, "Team Two Total");
    fixScore(dealOneScoreLabel, "Last Deal Team One");
    fixScore(dealTwoScoreLabel, "Last Deal Team Two");
    fixScore(hangingLabel, "Hanging");
    if (gameMode != GameManager.PLAYING && gameMode != GameManager.BETTING) {
      cleanGameState(gameMode);
    }
    startGameButton.setEnabled(gameMode == GameManager.NO_GAME);
    startDealingButton.setEnabled(gameMode == GameManager.NO_DEALING);
    connectButton.setEnabled(clientSenderThread == null);
    disconnectButton.setEnabled(clientSenderThread != null);
    fixPlayButton();
    fixAnnounces();
    fixBets();
  }

  private void doTimer() {
    long time = System.currentTimeMillis();
    if (handFullTimer != 0 && Math.abs(time - handFullTimer) >= TIMEOUT) {
      handFullTimer = 0;
      if (handOfCards[3] != null) {
        for (int i = 0; i < 4; ++i) {
          handOfCards[i] = null;
        }
      }
      fixHand();
    }
    for (int i = 0; i < 4; ++i) {
      if (announcesTimer[i] != 0 && Math.abs(time - announcesTimer[i]) >= TIMEOUT) {
        announcesTimer[i] = 0;
        announcesLabels[i].setText("");
      }
    }
  }

}