package org.belot.client;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import org.belot.protocol.*;
import org.belot.server.*;

import java.io.*;
import java.util.*;
import java.net.*;

public class BelotClientReader implements Runnable {

  private BelotClientImpl belotClient;

  private boolean mustFinish = false;

  private int playerId = -1000;
  private int gameState = -1;
  private int playerTurn = 0;
  private LineNumberReader in;
  private BelotCard[] playerCards = new BelotCard[8];

  private BelotColor currentTrump = null;
  private boolean isContra = false;
  private boolean isReContra = false;
  private int bettingPlayer = 300;

  private Vector players = new Vector();

  private long lastGetCardsId = -1;

  public BelotClientReader(BelotClientImpl belotClient, LineNumberReader in) {
    this.belotClient = belotClient;
    this.in = in;
    playerTurn = (0);
    new Thread(this).start();
  }

  private synchronized boolean finish() {
    return mustFinish;
  }

  public synchronized void setFinish() {
    this.mustFinish = true;
  }

  private void removePlayer(long id) {
    int size = players.size();
    for (int i = size - 1; i >= 0; --i) {
      BelotClientPlayer player = (BelotClientPlayer)players.elementAt(i);
      if (player.getId() == id) {
        players.removeElementAt(i);
      }
    }
  }

  private static BelotClientPlayer playerInfo(String[] parseResult) throws Exception {
    BelotServerProtocol.parse(parseResult[1], parseResult);
    long id = Long.parseLong(parseResult[0]);
    BelotServerProtocol.parse(parseResult[1], parseResult);
    String ip = parseResult[0];
    BelotServerProtocol.parse(parseResult[1], parseResult);
    String name = parseResult[0];
    BelotServerProtocol.parse(parseResult[1], parseResult);
    int direction = Integer.parseInt(parseResult[0]);
    return new BelotClientPlayer(name, ip, direction, id);
  }

  public void run() {
    try {
      String line;
      String[] parseResult = new String[2];
      while (!finish() && (line = in.readLine()) != null) {
        synchronized (this) {
          System.out.println("line = " + line);
          BelotServerProtocol.parse(line, parseResult);
          line = parseResult[1];
          String message = parseResult[0];
          if (BelotProtocol.GameStarted.equals(message)) {
            gameState = GameManager.NO_DEALING;
            playerTurn = (-10);
            setGameState();
          } else if (BelotProtocol.DealingStarted.equals(message)) {
            gameState = GameManager.BETTING;
            setGameState();
          } else if (BelotProtocol.YourId.equals(message)) {
            BelotServerProtocol.parse(line, parseResult);
            playerId = Integer.parseInt(parseResult[0]);
          } else if (BelotProtocol.CurrentPlayerBet.equals(message)) {
            gameState = GameManager.BETTING;
            BelotServerProtocol.parse(line, parseResult);
            playerTurn = (Integer.parseInt(parseResult[0]));
            setGameState();
          } else if (BelotProtocol.CurrentPlayerPlay.equals(message)) {
            gameState = GameManager.PLAYING;
            BelotServerProtocol.parse(line, parseResult);
            playerTurn = (Integer.parseInt(parseResult[0]));
            setGameState();
          } else if (BelotProtocol.NoGameAtPresent.equals(message)) {
            gameState = GameManager.NO_GAME;
            playerTurn = (-10);
            setGameState();
          } else if (BelotProtocol.NoDealingAtPresent.equals(message)) {
            gameState = GameManager.NO_DEALING;
            playerTurn = (-10);
            setGameState();
          } else if (BelotProtocol.YourCards.equals(message)) {
            playerCards = CardsParser.parseCards(parseResult, 8);
            setCards();
          } else if (BelotProtocol.CurrentTrump.equals(message)) {
            BelotServerProtocol.parse(line, parseResult);
            line = parseResult[1];
            currentTrump = BelotColor.getBetting(Integer.parseInt(parseResult[0]));
            BelotServerProtocol.parse(line, parseResult);
            line = parseResult[1];
            isContra = "true".equals(parseResult[0]);
            BelotServerProtocol.parse(line, parseResult);
            line = parseResult[1];
            isReContra = "true".equals(parseResult[0]);
            BelotServerProtocol.parse(line, parseResult);
            line = parseResult[1];
            bettingPlayer = Integer.parseInt(parseResult[0]);
            setTrump(currentTrump, isContra, isReContra, bettingPlayer);
          } else if (BelotProtocol.CurrentHand.equals(message)) {
            BelotCard[] currentHand = CardsParser.parseCards(parseResult, 4);
            System.out.println("left: " + parseResult[1]);
            BelotServerProtocol.parse(parseResult[1], parseResult);
            System.out.println("now: " + parseResult[0]);
            int firstPlayer = Integer.parseInt(parseResult[0]);
            setHand(currentHand, firstPlayer);
          } else if (BelotProtocol.PlayerList.equals(message)) {
            players = new Vector();
            while (parseResult[1].length() > 0) {
              BelotClientPlayer player = playerInfo(parseResult);
              players.addElement(player);
            }
            setPlayers();
          } else if (BelotProtocol.ActivePlayersList.equals(message)) {
            BelotClientPlayer[] players = new BelotClientPlayer[4];
            for (int i = 0; i < 4; ++i) {
              BelotClientPlayer player = playerInfo(parseResult);
              players[i] = player;
            }
            setActivePlayers(players);
          } else if (BelotProtocol.PlayerConnected.equals(message)) {
            BelotClientPlayer player = playerInfo(parseResult);
            removePlayer(player.getId());
            players.addElement(player);
            setPlayer(player);
          } else if (BelotProtocol.PlayerDisconnected.equals(message)) {
            BelotClientPlayer player = playerInfo(parseResult);
            removePlayer(player.getId());
            setRemovePlayer(player);
          } else if (BelotProtocol.PlayerChanged.equals(message)) {
            BelotClientPlayer player = playerInfo(parseResult);
            removePlayer(player.getId());
            players.addElement(player);
            setPlayer(player);
          } else if (BelotProtocol.PlayerBet.equals(message)) {
            BelotClientPlayer player = playerInfo(parseResult);
            BelotServerProtocol.parse(parseResult[1], parseResult);
            int color = Integer.parseInt(parseResult[0]);
            BelotServerProtocol.parse(parseResult[1], parseResult);
            boolean contra = "true".equals(parseResult[0]);
            BelotServerProtocol.parse(parseResult[1], parseResult);
            boolean reContra = "true".equals(parseResult[0]);
            setPlayerBet(player, color, contra, reContra);
          } else if (BelotProtocol.PlayerPlay.equals(message)) {
            BelotClientPlayer player = playerInfo(parseResult);
            BelotCard card = CardsParser.parseCards(parseResult, 1)[0];
            setPlayerPlay(player, card);
          } else if (BelotProtocol.PlayerPlayBelot.equals(message)) {
            BelotClientPlayer player = playerInfo(parseResult);
            BelotCard card = CardsParser.parseCards(parseResult, 1)[0];
            setPlayerPlayBelot(player, card);
          } else if (BelotProtocol.PlayerAnnounce.equals(message)) {
            BelotClientPlayer player = playerInfo(parseResult);
            BelotServerProtocol.parse(parseResult[1], parseResult);
            int color = Integer.parseInt(parseResult[0]);
            setPlayerAnnounce(player, color);
          } else if (BelotProtocol.Scores.equals(message)) {
            int[] dealingScores = new int[3];
            int[] teamScores = new int[2];
            for (int i = 0; i < 3; ++i) {
              BelotServerProtocol.parse(parseResult[1], parseResult);
              dealingScores[i] = Integer.parseInt(parseResult[0]);
            }
            for (int i = 0; i < 2; ++i) {
              BelotServerProtocol.parse(parseResult[1], parseResult);
              teamScores[i] = Integer.parseInt(parseResult[0]);
            }
            setScores(dealingScores, teamScores);
          } else if (BelotProtocol.Message.equals(message)) {
            BelotClientPlayer player = playerInfo(parseResult);
            setMessage(player, parseResult[1]);
          } else if (BelotProtocol.CommandResponse.equals(message)) {
            BelotServerProtocol.parse(parseResult[1], parseResult);
            long id = -100;
            try {
              id = Long.parseLong(parseResult[0]);
            } catch (Throwable error) {
              id = -100;
            }
            BelotServerProtocol.parse(parseResult[1], parseResult);
            boolean answer = "true".equals(parseResult[0]);
            if (!answer && id == getLastGetCardsId()) {
              playerCards = new BelotCard[8];
              setCards();
            }
          }
        }

      }
    } catch (Throwable error) {
      error.printStackTrace(System.out);
    } finally {
      setDisconnect();
    }
  }

  private void setGameState() {
    final int gameState = this.gameState;
    final int playerTurn = this.playerTurn;
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.gameMode(gameState, playerTurn);
        belotClient.fixEnabledState();
      }
    });
  }

  private void setCards() {
    final BelotCard[] playerCards = new BelotCard[8];
    for (int i = 0; i < 8; ++i) {
      playerCards[i] = this.playerCards[i];
    }
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.setCards(playerCards);
        belotClient.fixEnabledState();
      }
    });
  }

  private void setHand(final BelotCard[] currentHand, final int firstPlayer) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.setHand(currentHand, firstPlayer);
        belotClient.fixEnabledState();
      }
    });
  }

  private void setPlayers() {
    final Vector players = (Vector)this.players.clone();
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.listPlayers(players);
        belotClient.fixEnabledState();
      }
    });
  }

  private void setPlayer(final BelotClientPlayer player) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.listPlayer(player);
        belotClient.fixEnabledState();
      }
    });
  }

  private void setRemovePlayer(final BelotClientPlayer player) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.removePlayer(player);
        belotClient.fixEnabledState();
      }
    });
  }

  private void setPlayerBet(final BelotClientPlayer player, final int color, final boolean contra, final boolean reContra) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.setPlayerBet(player, color, contra, reContra);
        belotClient.fixEnabledState();
      }
    });
  }

  private void setPlayerPlay(final BelotClientPlayer player, final BelotCard card) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.setPlayerPlay(player, card);
        belotClient.fixEnabledState();
      }
    });
  }

  private void setPlayerPlayBelot(final BelotClientPlayer player, final BelotCard card) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.setPlayerPlayBelot(player, card);
        belotClient.fixEnabledState();
      }
    });
  }

  private void setPlayerAnnounce(final BelotClientPlayer player, final int what) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.setPlayerAnnounce(player, what);
        belotClient.fixEnabledState();
      }
    });
  }

  private void setTrump(final BelotColor trump, final boolean contra, final boolean recontra, final int player) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.setTrump(trump, contra, recontra, player);
        belotClient.fixEnabledState();
      }
    });
  }

  private void setDisconnect() {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.disconnect();
        belotClient.fixEnabledState();
      }
    });
  }

  private void setScores(final int[] dealingScores, final int[] teamScores) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.setScores(dealingScores, teamScores);
        belotClient.fixEnabledState();
      }
    });
  }

  private void setMessage(final BelotClientPlayer player, final String line) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.setPlayerMessageLine(player, line);
      }
    });
  }

  private void setActivePlayers(final BelotClientPlayer[] activePlayers) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        belotClient.setActivePlayers(activePlayers);
      }
    });
  }

  public synchronized long getLastGetCardsId() {
    return lastGetCardsId;
  }

  public synchronized long nextGetCardsId() {
    return ++lastGetCardsId;
  }

}