package org.belot.client.board;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import org.belot.protocol.*;
import org.belot.server.*;

import java.io.*;
import java.util.*;
import java.net.*;

import org.belot.client.*;
import org.belot.client.images.*;

public abstract class HandPanel extends JPanel {

  public static final int SOUTH = 1;

  protected static final int INSETS = 5;

  protected JLabel westLabel = new JLabel();
  protected JLabel southLabel = new JLabel();
  protected JLabel eastLabel = new JLabel();
  protected JLabel northLabel = new JLabel();
  protected JLabel[] labelsByPosition = new JLabel[] {westLabel, southLabel, eastLabel, northLabel};
  protected ImageIcon[] imagesByPosition = new ImageIcon[4];
  protected CardsImages cardsImages;
  protected Insets insets = new Insets(INSETS, INSETS, INSETS, INSETS);
  protected BelotClient belotClient;

  public HandPanel(CardsImages cardsImages, BelotClient belotClient) {
    super(new GridBagLayout());

//    this.setBackground(Color.white);
//
    this.cardsImages = cardsImages;
    this.belotClient = belotClient;

    for (int i = 0; i < 4; ++i) {
      setLabelSizes(labelsByPosition[i]);
    }

    // first col:
    this.add(westLabel, new GridBagConstraints(0, 0, 1, 2, 0.0, 0.0,
      GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));

    // second col:
    this.add(northLabel, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
      GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
    this.add(southLabel, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
      GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));

    // third col:
    this.add(eastLabel, new GridBagConstraints(2, 0, 1, 2, 0.0, 0.0,
      GridBagConstraints.CENTER, GridBagConstraints.NONE, insets, 0, 0));
  }

  public static void setLabelSizes(JLabel label) {
    label.setText("");
    label.setIconTextGap(0);
    label.setSize(CardsImages.CardSize);
    label.setMinimumSize(CardsImages.CardSize);
    label.setMaximumSize(CardsImages.CardSize);
    label.setPreferredSize(CardsImages.CardSize);
  }

  public Dimension getPreferredSize() {
    return new Dimension(
      CardsImages.CARD_WIDTH * 3 + 6 * INSETS,
      CardsImages.CARD_HEIGHT * 2 + 4 * INSETS
    );
  }

  public abstract void loadCards(BelotCard[] handOfCards, int playerPosition, int firstHandPlayer);

  public static void main(String[] arg) {
    CardsImages cardsImages = new CardsImages();
    HandPanel boardPanel = new HandPanelImpl(cardsImages, null);
    JFrame frame = new JFrame();
    JScrollPane scrollPane = new JScrollPane(boardPanel);
    frame.getContentPane().add(scrollPane);
    Dimension size = new Dimension(800, 600);
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    frame.setLocation((screenSize.width - size.width) / 2, (screenSize.height - size.height) / 2);
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setSize(size);
    boardPanel.loadCards(new BelotCard[] {
      new BelotCard(0, 11),
      new BelotCard(1, 12),
      new BelotCard(2, 13),
      null,
    }, 2, 1);
    frame.setVisible(true);
  }

}