package org.belot.client.images;

import java.io.*;
import java.net.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;
import org.belot.protocol.*;
import org.belot.server.*;

public class CardsImages {

  public static final int CARD_HEIGHT = 96;
  public static final int CARD_WIDTH = 71;

  public static final Dimension CardSize = new Dimension(CARD_WIDTH, CARD_HEIGHT);

  private Hashtable images = new Hashtable();
  private Hashtable missing = new Hashtable();

  private byte[] loadFile(String name) {
    System.out.println("name = " + name);
    InputStream in = null;
    try {
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      in = new BufferedInputStream(getClass().getResourceAsStream(name), 1024 * 8);
      byte[] bytes = new byte[1024 * 8];
      int n;
      while ((n = in.read(bytes)) > -1) {
        out.write(bytes, 0, n);
      }
      out.close();
      return out.toByteArray();
    } catch (Throwable error) {
//      error.printStackTrace(System.out);
      return null;
    } finally {
      if (in != null) {
        try {
          in.close();
        } catch (Throwable aThrowable) {
          // do nothing.
        }
      }
    }
  }

  private ImageIcon loadImage(String key) {
    if (missing.get(key) != null) {
      return null;
    }
    ImageIcon image = (ImageIcon)images.get(key);
    if (image == null) {
      byte[] imageData = loadFile(key);
      if (imageData != null && imageData.length > 0) {
        image = new ImageIcon(imageData);
        images.put(key, image);
      } else {
        missing.put(key, key);
      }
    }
    return image;
  }

  public ImageIcon getImage(BelotColor color) {
    if (color == null) {
      return null;
    }
    String key = color.getName() + ".png";
    return loadImage(key);
  }

  public ImageIcon getImage(BelotCard card) {
    if (card == null) {
      return null;
    }
    String key = card.getColor().getName() + "_" + card.getNumber() + ".png";
    return loadImage(key);
  }

  public ImageIcon getBack() {
    return loadImage("Back.png");
  }

  public ImageIcon getBall() {
    return loadImage("Ball.png");
  }

}