package org.belot.server;

import org.belot.protocol.*;
import java.util.*;

public class Announces implements Comparator {

  private Vector announces = new Vector();

  private static int[] scores = new int[] {
    20, 50, 100, 100, 150, 200
  };

  public static int getScore(int what) {
    return scores[what - BelotProtocol.AnnounceSequence3];
  }

  public boolean canAdd(int what, BelotCard[] cards, int card) {
    if (cards == null || cards.length != 8) {
      return false;
    }
    for (int i = 0; i < cards.length; ++i) {
      if (cards[i] == null) {
        return false;
      }
    }
    if (isInOther(cards, card)) {
      return false;
    }
    switch (what) {
      case (BelotProtocol.AnnounceSequence3) : {
        return hasCards(3, cards, card);
      }
      case (BelotProtocol.AnnounceSequence4) : {
        return hasCards(4, cards, card);
      }
      case (BelotProtocol.AnnounceSequence5) : {
        return hasCards(5, cards, card);
      }
      case (BelotProtocol.AnnounceFourOther) :
      case (BelotProtocol.AnnounceFourNines) :
      case (BelotProtocol.AnnounceFourJacks) : {
        return hasFourSame(cards, card);
      }
    }
    return false;
  }

  private boolean isInOther(BelotCard[] cards, int card) {
    int size = announces.size();
    for (int i = 0; i < size; ++i) {
      int[] announce = (int[])announces.elementAt(i);
      int announceWhat = announce[0];
      int announceCard = announce[1];
      switch (announceWhat) {
        case (BelotProtocol.AnnounceSequence3) : {
          if (isInSequence(announceCard, 3, cards, card)) return true;
          break;
        }
        case (BelotProtocol.AnnounceSequence4) : {
          if (isInSequence(announceCard, 4, cards, card)) return true;
          break;
        }
        case (BelotProtocol.AnnounceSequence5) : {
          if (isInSequence(announceCard, 5, cards, card)) return true;
          break;
        }
        default : {
          if (isInFourSame(announceCard, cards, card)) return true;
          break;
        }
      }
    }
    return false;
  }

  private static boolean isInSequence(int announceCard, int number, BelotCard[] cards, int card) {
    BelotColor color = cards[announceCard].getColor();
    if (cards[card].getColor() != color) {
      return false;
    }
    int to = cards[announceCard].getNumber();
    int from = (to - number) + 1;
    int newNumber = cards[card].getNumber();
    return newNumber >= from && newNumber <= to;
  }

  private static boolean isInFourSame(int announceCard, BelotCard[] cards, int card) {
    return cards[announceCard].getNumber() == cards[card].getNumber();
  }

  private static boolean hasFourSame(BelotCard[] cards, int card) {
    int number = cards[card].getNumber();
    if (number < 9) return false;
    for (int i = 0; i < 4; ++i) {
      if (!findCard(BelotColor.getColor(i), number, cards)) {
        return false;
      }
    }
    return true;
  }

  private static boolean hasCards(int number, BelotCard[] cards, int card) {
    BelotColor color = cards[card].getColor();
    int to = cards[card].getNumber();
    if (to < 9) {
      return false;
    }
    int from = (to - number) + 1;
    // the card at 'to' is indeed there!
    // so don't check for it!
    for (int i = from; i < to; ++i) {
      if (!findCard(color, i, cards)) {
        return false;
      }
    }
    return true;
  }

  public static boolean findCard(BelotColor color, int number, BelotCard[] cards) {
    for (int i = 0; i < cards.length; ++i) {
      if (cards[i] != null && cards[i].getColor() == color && cards[i].getNumber() == number) {
        return true;
      }
    }
    return false;
  }

  public void add(int what, BelotCard[] cards, int card) {
    announces.addElement(new int[] {what, card, cards[card].getNumber()});
  }

  public void remove(int what, int card) {
    int size = announces.size();
    for (int i = 0; i < size; ++i) {
      int[] announce = (int[])announces.elementAt(i);
      if (announce[0] == what && announce[1] == card) {
        announces.removeElementAt(i);
        break;
      }
    }
  }

  public int compare(Object x, Object y) {
    if (x == y) return 0;
    if (x == null) return -1;
    if (y == null) return 1;
    int[] a = (int[])x;
    int[] b = (int[])y;
    // 1. compare the value:
    int cmp = getScore(a[0]) - getScore(b[0]);
    if (cmp != 0) return cmp;
    // 2. compare the card number:
    return a[2] - b[2];
  }

  public void addToVector(Vector vec) {
    int size = announces.size();
    for (int i = 0; i < size; ++i) {
      vec.addElement(announces.elementAt(i));
    }
  }

  public Vector get() {
    return announces;
  }

  public void clear() {
    announces.removeAllElements();
  }

}