package org.belot.server;

public class BelotCard {

  public static final int NUMBER_JACK = 11;
  public static final int NUMBER_QUEEN = 12;
  public static final int NUMBER_KING = 13;
  public static final int NUMBER_ACE = 14;

  private static final String[] FullNames = new String[] {
    "Seven",
    "Eight",
    "Nine",
    "Ten",
    "Jack",
    "Queen",
    "King",
    "Ace",
  };

  private static final String[] ShortNames = new String[] {
    "7",
    "8",
    "9",
    "10",
    "J",
    "Q",
    "K",
    "A",
  };

  private static final int[] WeightTrump = new int[] {
    7, 8, NUMBER_QUEEN, NUMBER_KING, 10, NUMBER_ACE, 9, NUMBER_JACK
  };

  private static final int[] WeightNonTrump = new int[] {
    7, 8, 9, NUMBER_JACK, NUMBER_QUEEN, NUMBER_KING, 10, NUMBER_ACE
  };

  private static final int[] ScoreTrump = new int[] {
    /*7 */  0,
    /*8 */  0,
    /*9 */  14,
    /*10*/  10,
    /*J */  20,
    /*Q */  3,
    /*K */  4,
    /*A */  11,
  };

  private static final int[] ScoreNonTrump = new int[] {
    /*7 */  0,
    /*8 */  0,
    /*9 */  0,
    /*10*/  10,
    /*J */  2,
    /*Q */  3,
    /*K */  4,
    /*A */  11,
  };

  private BelotColor color;
  private int number;

  public BelotCard(int color, int number) {
    this(BelotColor.getColor(color), number);
  }

  public BelotCard(BelotColor color, int number) {
    if (color.getColor() >= BelotColor.COLOR_NOTRUMP || number < 7 || number > NUMBER_ACE) {
      throw new RuntimeException("Invalid card: " + number);
    }
    this.color = color;
    this.number = number;
  }

  public String getFullName() {
    return FullNames[number - 7] + " of " + color.getName();
  }

  public String toString() {
    return getFullName();
  }

  public BelotColor getColor() {
    return color;
  }

  public static int findIndex(int number, int[] numbers) {
    for (int i = 0; i < numbers.length; ++i) {
      if (number == numbers[i]) {
        return i;
      }
    }
    return -1;
  }

  public int compareTo(int otherNumber, BelotColor trump) {
    if (this.number == otherNumber) {
      return 0;
    }
    int[] weights;
    if (trump == BelotColor.AllTrump || color == trump) {
      weights = WeightTrump;
    } else {
      weights = WeightNonTrump;
    }
    int indexThis = findIndex(this.number, weights);
    int indexOther = findIndex(otherNumber, weights);
    return indexThis - indexOther;
  }

  public int getScore(BelotColor trump) {
    int index = this.number - 7;
    if (trump == BelotColor.AllTrump || color == trump) {
      return ScoreTrump[index];
    }
    return ScoreNonTrump[index];
  }

  public static void main(String[] arg) {
    BelotCard card1 = new BelotCard(BelotColor.Clubs, NUMBER_JACK);
    System.out.println(card1.compareTo(NUMBER_ACE, BelotColor.Clubs));
  }

  public int getNumber() {
    return number;
  }

}