package org.belot.server;

public final class BelotColor {

  public static final int COLOR_ALLTRUMP = 5;
  public static final int COLOR_NOTRUMP = 4;

  public static final int COLOR_SPADES = 3;
  public static final int COLOR_HEARTS = 2;
  public static final int COLOR_DIAMONDS = 1;
  public static final int COLOR_CLUBS = 0;

  public static final BelotColor AllTrump = new BelotColor(COLOR_ALLTRUMP, "All Trump");
  public static final BelotColor NoTrump = new BelotColor(COLOR_NOTRUMP, "No Trump");

  public static final BelotColor Spades = new BelotColor(COLOR_SPADES, "Spades");
  public static final BelotColor Hearts = new BelotColor(COLOR_HEARTS, "Hearts");
  public static final BelotColor Diamonds = new BelotColor(COLOR_DIAMONDS, "Diamonds");
  public static final BelotColor Clubs = new BelotColor(COLOR_CLUBS, "Clubs");

  private int color;
  private String name;

  private BelotColor(int color, String name) {
    this.color = color;
    this.name = name;
    if (color < 0 || color > 5) throw new RuntimeException("Invalid color: " + color);
  }

  public int getColor() {
    return color;
  }

  public int compareTo(BelotColor other) {
    return color - other.color;
  }

  public static BelotColor getBetting(int color) {
    if (color < 0) {
      return null;
    }
    if (color < COLOR_NOTRUMP) {
      return getColor(color);
    }
    if (color == COLOR_NOTRUMP) {
      return NoTrump;
    }
    if (color == COLOR_ALLTRUMP) {
      return AllTrump;
    }
    return null;
  }

  public static BelotColor getColor(int color) {
    switch (color) {
      case (COLOR_SPADES) : {
        return Spades;
      }
      case (COLOR_HEARTS) : {
        return Hearts;
      }
      case (COLOR_DIAMONDS) : {
        return Diamonds;
      }
      case (COLOR_CLUBS) : {
        return Clubs;
      }
      default : {
        // fall through:
        break;
      }
    }
    throw new RuntimeException("Invalid color: " + color);
  }

  public String getName() {
    return name;
  }

  public static void main(String[] arg) {
    System.out.println(Diamonds.compareTo(Hearts));
  }

}