package org.belot.server;

public class BelotDeck {

  public static final int CARDS = 32;
  public static final int CARDS_PER_PLAYER = CARDS / 4;

  private BelotCard[] cards;
  private int left;

  public BelotDeck() {
    cards = new BelotCard[CARDS];
    for (int i = 0; i < CARDS; ++i) {
      int color = i / (CARDS / 4);
      int number = i % (CARDS / 4);
      cards[i] = new BelotCard(color, number + 7);
    }
  }

  public int deal(BelotCard[] result, int firstIndex, int n) {
    int toDeal = Math.min(left, n);
    for (int i = 0; i < toDeal; i++) {
      result[i + firstIndex] = cards[i + (CARDS - left)];
    }
    left -= toDeal;
    return toDeal;
  }

  public int getLeft() {
    return left;
  }

  public void shuffle() {
    for (int i = 0; i < CARDS; ++i) {
      // int between 0 and i:
      int r = (int)(Math.random() * (i + 1));
      if (r > i) {
        r = i;
      }
      if (r < 0) {
        r = 0;
      }
      BelotCard swap = cards[r];
      cards[r] = cards[i];
      cards[i] = swap;
    }
    left = CARDS;
  }

  private static void printCards(BelotCard[] cards) {
    for (int i = 0; i < cards.length; ++i) {
      System.out.println(cards[i]);
    }
  }

  public void print() {
    printCards(cards);
  }

  public static void main(String[] arg) {
    BelotDeck deck = new BelotDeck();
    deck.shuffle();
    BelotCard[] cards = new BelotCard[CARDS_PER_PLAYER];
    System.out.println("-------------");
    deck.print();
    System.out.println("-------------");
    deck.deal(cards, 0, 3);
    deck.deal(cards, 3, 2);
    deck.deal(cards, 5, 3);
    printCards(cards);
    System.out.println("-------------");
  }

}