package org.belot.server;

import org.belot.protocol.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class BelotServerProtocol {

  public static final String FirstResponse = BelotProtocol.CommandResponse + ":" + 0 + ":" + "true" + ":" + "N/A";

  public static String yourId(BelotPlayer player) {
    //YourId
    return BelotProtocol.YourId + ":" + player.getId();
  }

  private static String playerInfo(BelotPlayer player) {
    return playerInfo(player, -1);
  }

  private static String playerInfo(BelotPlayer player, int backupDirection) {
    if (player != null) {
      return player.getId() + ":" + player.getIp() + ":" + player.getName() + ":" + player.getDirection();
    } else {
      return "-1:127.0.0.1:null:" + backupDirection;
    }
  }

  public static String playerConnected(BelotPlayer player) {
    return BelotProtocol.PlayerConnected + ":" + playerInfo(player);
  }

  public static String playerDisconnected(BelotPlayer player) {
    return BelotProtocol.PlayerDisconnected + ":" + playerInfo(player);
  }

  public static String playerChanged(BelotPlayer player) {
    return BelotProtocol.PlayerChanged + ":" + playerInfo(player);
  }

  public static void parse(String line, String[] parseResult) {
    int index = line.indexOf(':');
    if (index >= 0) {
      parseResult[0] = line.substring(0, index).trim();
      parseResult[1] = line.substring(index + 1, line.length());
    } else {
      parseResult[0] = line.trim();
      parseResult[1] = "";
    }
  }

  public static String commandResponse(String commandId, boolean commandResponse, String answer) {
    return
      BelotProtocol.CommandResponse + ":" +
      commandId + ":" +
      (commandResponse ? "true" : "false") + ":" +
      answer;
  }

  public static String listPlayers(Enumeration players) {
    StringBuffer result = new StringBuffer();
    result.append(BelotProtocol.PlayerList);
    while (players.hasMoreElements()) {
      BelotPlayer player = (BelotPlayer)players.nextElement();
      result.append(":").append(playerInfo(player));
    }
    return result.toString();
  }

  public static String activePlayers(PlayerManager playerManager) {
    StringBuffer result = new StringBuffer();
    result.append(BelotProtocol.ActivePlayersList);
    for (int i = 0; i < 4; ++i) {
      BelotPlayer player = playerManager.getPlayer(i);
      result.append(":").append(playerInfo(player, i));
    }
    return result.toString();
  }

  public static String trump(GameManager gameManager) {
    BelotColor trump = gameManager.getBet();
    return
      BelotProtocol.CurrentTrump + ":" +
      (trump == null ? -1 : trump.getColor()) + ":" +
      gameManager.isContra() + ":" +
      gameManager.isReContra() + ":" +
      gameManager.getBettingPlayer();
  }

  public static String bet(BelotPlayer belotPlayer, BelotColor color, boolean contra, boolean reContra) {
    return
      BelotProtocol.PlayerBet + ":" +
      playerInfo(belotPlayer) + ":" +
      (color != null ? color.getColor() : -1) + ":" +
      contra + ":" + reContra;
  }

  private static String cardInfo(BelotCard card) {
    if (card == null) {
      return "null:-1:-1";
    }
    return card.toString() + ":" + card.getNumber() + ":" + card.getColor().getColor();
  }

  public static String play(BelotPlayer belotPlayer, BelotCard card) {
    return
      BelotProtocol.PlayerPlay + ":" +
      playerInfo(belotPlayer) + ":" +
      cardInfo(card);
  }

  public static String playBelot(BelotPlayer belotPlayer, BelotCard card) {
    return
      BelotProtocol.PlayerPlayBelot + ":" +
      playerInfo(belotPlayer) + ":" +
      cardInfo(card);
  }

  public static String betTurn(int direction) {
    return BelotProtocol.CurrentPlayerBet + ":" + direction;
  }

  public static String playTurn(int direction) {
    return BelotProtocol.CurrentPlayerPlay + ":" + direction;
  }

  private static String listCards(String message, BelotCard[] cards) {
    StringBuffer buf = new StringBuffer();
    buf.append(message);
    for (int i = 0; i < cards.length; ++i) {
      buf.append(":").append(cardInfo(cards[i]));
    }
    return buf.toString();
  }

  public static String listCards(BelotCard[] cards) {
    return listCards(BelotProtocol.YourCards, cards);
  }

  public static String currentHand(BelotCard[] cards, int firstPlayer) {
    return listCards(BelotProtocol.CurrentHand, cards) + ":" + firstPlayer;
  }

  public static String scores(GameManager gameManager) {
    int[] dealingScores = gameManager.getDealingScores();
    int[] teamScores = gameManager.getTeamScores();
    return BelotProtocol.Scores + ":" +
      dealingScores[0] + ":" +
      dealingScores[1] + ":" +
      dealingScores[2] + ":" +
      teamScores[0] + ":" +
      teamScores[1];
  }

  public static String playerAnnounce(BelotPlayer belotPlayer, int what) {
    return
      BelotProtocol.PlayerAnnounce + ":" +
      playerInfo(belotPlayer) + ":" +
      what;
  }

  public static String message(BelotPlayer belotPlayer, String line) {
    return
      BelotProtocol.Message + ":" +
      playerInfo(belotPlayer) + ":" +
      line;
  }

}