package org.belot.server;

import java.util.*;

public class PlayerManager {

  private Vector players;
  private BelotPlayer[] activePlayers;

  public PlayerManager() {
    synchronized (this) {
      this.players = new Vector();
      this.activePlayers = new BelotPlayer[4];
    }
  }

  // Player management:
  public synchronized void addPlayer(BelotPlayer belotPlayer) {
    players.addElement(belotPlayer);
  }

  public synchronized void removePlayer(BelotPlayer belotPlayer) {
    players.removeElement(belotPlayer);
    cleanPlayer(belotPlayer);
  }

  private void cleanPlayer(BelotPlayer belotPlayer) {
    for (int i = 0; i < 4; ++i) {
      if (this.activePlayers[i] == belotPlayer) {
        this.activePlayers[i] = null;
      }
    }
  }

  public synchronized void changePlayer(BelotPlayer belotPlayer) {
    cleanPlayer(belotPlayer);
    fillGame();
  }

  public synchronized int size() {
    return players.size();
  }

  // List players:
  public synchronized Enumeration getPlayers() {
    return players.elements();
  }

  public synchronized BelotPlayer getPlayer(int position) {
    if (position >= 0 && position <= 3) {
      return activePlayers[position];
    }
    return null;
  }

  private int findPlayerNumber(int number) {
    int size = players.size();
    for (int i = 0; i < size; ++i) {
      BelotPlayer player = (BelotPlayer)players.elementAt(i);
      if (player.getDirection() == number) {
        return i;
      }
    }
    return -1;
  }

  public synchronized void fillGame() {
    for (int i = 0; i < 4; ++i) {
      if (activePlayers[i] == null) {
        int index = findPlayerNumber(i);
        if (index != -1) {
          activePlayers[i] = (BelotPlayer)players.elementAt(index);
        }
      }
    }
  }

  public synchronized boolean hasAllPlayers() {
    for (int i = 0; i < 4; ++i) {
      if (activePlayers[i] == null) {
        return false;
      }
    }
    return true;
  }

  public synchronized void endGame() {
    for (int i = 0; i < 4; ++i) {
      activePlayers[i] = null;
    }
  }

}