package org.belot.test;

import java.io.*;
import java.net.*;
import org.belot.protocol.*;
import org.belot.server.*;

public class BelotTest implements Runnable {

  private int playerNumber;
  private long commandId = 1;
  private int playCard = 0;
  private ClientSenderThread clientSenderThread;

  public BelotTest(int playerNumber) {
    this.playerNumber = playerNumber;
  }

  public void run() {
    try {
      Socket socket = new Socket("localhost", BelotProtocol.BelotPort);
      LineNumberReader in = new LineNumberReader(new InputStreamReader(socket.getInputStream(), "UTF8"));
      clientSenderThread = new ClientSenderThread(socket.getOutputStream());
      clientSenderThread.depositMessage(":" + BelotProtocol.Direction + ":" + playerNumber);
      clientSenderThread.depositMessage(":" + BelotProtocol.Name + ":" + "MyNameIs" + playerNumber);
      BelotTestReader reader = new BelotTestReader(in);
      for (;;) {
        play(reader.getGameState(), reader.getPlayerTurn());
        Thread.yield();
      }
    } catch (Throwable error) {
      error.printStackTrace(System.out);
    }
  }

  private void play(int gameState, int playerTurn) throws Exception {
    if (playerTurn != playerNumber) {
      return;
    }
    if (playerNumber < 0) {
      return;
    }
//    if (Math.random() < 0.4) {
//      return;
//    }
    String command = null;
    switch (gameState) {
      case (GameManager.NO_GAME) : {
        command = ("" + commandId++ + ":" + BelotProtocol.StartGame);
        break;
      } case (GameManager.NO_DEALING) : {
        command = ("" + commandId++ + ":" + BelotProtocol.StartDealing);
        break;
      } case (GameManager.BETTING) : {
        if (playerNumber == 0) {
          command = ("" + commandId++ + ":" + BelotProtocol.TrumpChoice + ":" + 5);
        } else {
          command = ("" + commandId++ + ":" + BelotProtocol.TrumpChoice + ":" + -1);
        }
        playerTurn = -1;
        break;
      } case (GameManager.PLAYING) : {
        command = ("" + commandId++ + ":" + BelotProtocol.Play + ":" + playCard);
        playCard = (playCard + 1) % 8;
        break;
      }
    }
    if (command != null) {
      System.out.println("---------->  player: " + playerNumber + " -> " + command);
      clientSenderThread.depositMessage(command);
    }
  }

  public static void main(String[] arg) {
    for (int i = 0; i < 4; ++i) {
      new Thread(new BelotTest(i)).start();
    }
  }

}