package org.belot.test;

import java.io.*;
import java.net.*;
import org.belot.protocol.*;
import org.belot.server.*;

public class BelotTestReader implements Runnable {

  private int playerNumber = -1000;
  private int gameState = -1;
  private int playerTurn = 0;
  private LineNumberReader in;

  public BelotTestReader(LineNumberReader in) {
    this.in = in;
    new Thread(this).start();
  }

  public void run() {
    try {
      String line;
      String[] parseResult = new String[2];
      while ((line = in.readLine()) != null) {
        if (playerNumber == 0) {
          System.out.println(line);
        }

        synchronized (this) {
          BelotServerProtocol.parse(line, parseResult);
          line = parseResult[1];
          String message = parseResult[0];
          if (BelotProtocol.GameStarted.equals(message)) {
            gameState = GameManager.NO_DEALING;
            playerTurn = 0;
          } else if (BelotProtocol.DealingStarted.equals(message)) {
            gameState = GameManager.BETTING;
          } else if (BelotProtocol.YourId.equals(message)) {
            BelotServerProtocol.parse(line, parseResult);
            playerNumber = Integer.parseInt(parseResult[0]);
          } else if (BelotProtocol.CurrentPlayerBet.equals(message)) {
            gameState = GameManager.BETTING;
            BelotServerProtocol.parse(line, parseResult);
            playerTurn = Integer.parseInt(parseResult[0]);
          } else if (BelotProtocol.CurrentPlayerPlay.equals(message)) {
            gameState = GameManager.PLAYING;
            BelotServerProtocol.parse(line, parseResult);
            playerTurn = Integer.parseInt(parseResult[0]);
          } else if (BelotProtocol.NoGameAtPresent.equals(message)) {
            gameState = GameManager.NO_GAME;
          } else if (BelotProtocol.NoDealingAtPresent.equals(message)) {
            gameState = GameManager.NO_DEALING;
          }
        }

      }
    } catch (Throwable error) {
      error.printStackTrace(System.out);
    }
  }

  public synchronized int getGameState() {
    return gameState;
  }

  public synchronized int getPlayerTurn() {
    return playerTurn;
  }

  public synchronized int getPlayerNumber() {
    return playerNumber;
  }

}